﻿#pragma once
#include <condition_variable>
#include <mutex>
#include <thread>

class thread_event {
public:
    thread_event() = default;
    explicit thread_event(bool manual_reset) : manual_reset(manual_reset) {}

    void wait() {
        std::unique_lock lock(mutex);
        cv.wait(lock, [this] { return signaled; });
        if (!manual_reset) {
            signaled = false;
        }
    }

    bool wait(std::chrono::milliseconds timeout) {
        std::unique_lock lock(mutex);
        const bool was_signaled = cv.wait_for(lock, timeout, [this] { return signaled; });
        if (was_signaled && !manual_reset) {
            signaled = false;
        }
        return was_signaled;
    }

    void signal() {
        std::lock_guard lock(mutex);
        signaled = true;
        if (manual_reset) {
            cv.notify_all();
        } else {
            cv.notify_one();
        }
    }

    // 重置事件为无信号状态（仅手动复位事件使用）
    void reset() {
        if (manual_reset) {
            std::lock_guard lock(mutex);
            signaled = false;
        }
    }
private:
    std::condition_variable cv;
    std::mutex mutex;
    bool signaled = false;
    bool manual_reset = false;
};
