﻿#include "thread_pool.h"

#include <iostream>

thread_pool::thread_pool(const size_t num_threads) : stop(false) {
    // 创建线程
    for (std::size_t i = 0; i < num_threads; ++i) {
        threads.emplace_back(&thread_pool::worker_thread, this);
    }
}

thread_pool::~thread_pool() {
    stop = true;
    // 唤醒所有线程
    condition.notify_all();

    for (auto& thread : threads) {
        if (thread.joinable()) {
            thread.join();
        }
    }
}

void thread_pool::worker_thread() {
    while (true) {
        std::function<void()> task;
        {
            std::unique_lock lock(queue_mutex);
            condition.wait(lock, [this] { return stop || !this->tasks.empty(); });
            if (stop && tasks.empty()) {
                return;
            }
            task = std::move(tasks.front());
            tasks.pop();
        }

        try {
            task();
        } catch (const std::exception& e) {
            // 处理任务中的异常，避免线程池崩溃
            std::cerr << "Task exception: " << e.what() << std::endl;
        } catch (...) {
            std::cerr << "Task exception: unknown exception" << std::endl;
        }
    }
}
