﻿#pragma once
#include <cstdint>
#include <Eigen/Eigen>

#include "font_type.h"
#include "containers/lrucache.hpp"

struct glyph_key {
	font_family family{};
	char32_t code_point{};
	float font_size{};
	float dpi_scale{};

	bool operator==(const glyph_key& other) const {
		return family == other.family &&
		       code_point == other.code_point &&
		       font_size == other.font_size &&
		       dpi_scale == other.dpi_scale;
	}
};

template <>
struct std::hash<glyph_key> {
	size_t operator()(const glyph_key& in_key) const noexcept {
		size_t hash = 0;
		hash_combine(hash, in_key.family.get());
		hash_combine(hash, in_key.code_point);
		hash_combine(hash, in_key.font_size);
		hash_combine(hash, in_key.dpi_scale);
		return hash;
	}
private:
	template<class T>
	static void hash_combine(size_t& in_seed, const T& in_v) {
		std::hash<T> hasher;
		in_seed ^= hasher(in_v) + 0x9e3779b9 + (in_seed << 6) + (in_seed >> 2);
	}
};

struct glyph_metadata {
	// LLGL::Texture* atlas_texture;
	Eigen::AlignedBox2f uv_rect;
	Eigen::Vector2f size;
	Eigen::Vector2f bearing;
};

class glyph_cache {
public:
	~glyph_cache();

	void initialize();
	void clear();
	glyph_metadata get_glyph(const glyph_key& in_key);
private:
	struct atlas_page {
		// LLGL::Texture* texture = nullptr;
		// LLGL::RenderSystem* render_system = nullptr;
		uint32_t current_x = 0;
		uint32_t current_y = 0;
		uint32_t row_height = 0;

		bool put_glyph(Eigen::AlignedBox2f& out_uv_rect);
	};

	cache::lru_cache<glyph_key, glyph_metadata> metadata_cache{500};
	std::vector<atlas_page> active_atlases;
	std::mutex atlas_mutex;
	// LLGL::RenderSystem* render_system = nullptr;

	atlas_page allocate_space(uint32_t in_width, uint32_t in_height);
	glyph_metadata make_glyph(const glyph_key& in_key);
};

