﻿#pragma once
#include <stdexcept>

class lazy_singleton_func {
protected:
	template<class T> static void construct(void* in_place) { new (in_place) T(); }
	template<class T> static void destruct(T* in_instance) { in_instance->~T(); }
};

template<class T>
class lazy_singleton : public lazy_singleton_func {
public:
	static T& get() {
		return *get_lazy(construct<T>).ptr;
	}

	static T* try_get() {
		return get_lazy(construct<T>).try_get_value();
	}

	static void tear_down() {
		get_lazy(nullptr).reset();
	}

private:
	static lazy_singleton& get_lazy(void(*ctor)(void*)) {
		static lazy_singleton instance(ctor);
		return instance;
	}

	explicit lazy_singleton(void(*ctor)(void*)) {
		if (ctor) {
			ctor(instance_data);
		}

		ptr = ctor ? reinterpret_cast<T*>(instance_data) : nullptr;
	}

	~lazy_singleton() {
		reset();
	}

	T* try_get_value() {
		return ptr;
	}

	T& get_value() {
		if (!ptr) {
			throw std::runtime_error("lazy_singleton not initialized");
		}
		return *ptr;
	}

	void reset() {
		if (ptr) {
			destruct(ptr);
			ptr = nullptr;
		}
	}

	alignas(T) unsigned char instance_data[sizeof(T)]{};
	T* ptr{};
};
