#pragma once
#include <vector>
#include <vulkan/vulkan.hpp>
#include <spdlog/spdlog.h>

class device_selector {
    enum class device_type { combined, display, compute, unknown };
public:
    using device_group = std::vector<vk::PhysicalDevice>;

    void classify_and_sort_devices(vk::Instance instance, vk::SurfaceKHR surface);

    vk::PhysicalDevice get_main_device() const;

    vk::PhysicalDevice get_main_compute_device() const;

    [[nodiscard]] static std::vector<uint32_t> get_queue_families(vk::PhysicalDevice in_physical_device, const vk::QueueFlags& in_flags);

    device_group combined_group; // 同时支持显示和计算的设备组
    device_group display_group;  // 显示设备组
    device_group compute_group;  // 计算设备组
private:
    using device_score = std::pair<vk::PhysicalDevice, int>;

    static std::vector<device_score> get_device_scores(const std::vector<vk::PhysicalDevice>& devices, vk::SurfaceKHR surface);

    static int score_device(const vk::PhysicalDevice& dev, vk::SurfaceKHR surface);

    void group_devices(const std::vector<device_score>& deviceScores, vk::SurfaceKHR surface);

    static device_type classify_device(const vk::PhysicalDevice& dev, vk::SurfaceKHR surface);
};
