﻿#pragma once
#include <chrono>
#include <filesystem>
#include <vulkan/vulkan.hpp>
#include <GLFW/glfw3.h>

#include "device_selector.h"
#include "renderer_swapchain.h"
#include "renderer_types.h"

struct window_desc;

class aorii_renderer {
public:
    bool init();
    void destroy();

    vk::SurfaceKHR create_surface(GLFWwindow* in_window) const;
    void destroy_surface(const vk::SurfaceKHR& in_surface) const;
    device_handle get_main_device() const { return main_device; }
    const std::vector<device_handle>& get_compute_device() const;

    const auto& get_instance() const { return instance; }
private:
    vk::Instance instance;
    device_selector selector;
    device_handle main_device;
    mutable std::vector<device_handle> compute_device;
};

namespace aorii {
    aorii_renderer* get_renderer();

    bool create_renderer();
    void destroy_renderer();

    inline std::filesystem::path s_shader_relative_path = "resource/shaders";
    inline void set_shader_relative_path(const std::filesystem::path& path) { s_shader_relative_path = path; }
    inline std::filesystem::path get_shader_path(const std::string& shader_name) { return std::filesystem::current_path() / s_shader_relative_path / shader_name; }
}
