#pragma once
#include <vulkan/vulkan.hpp>
#include <memory>

#include "renderer_types.h"

class buffer_memory_strategy {
public:
    virtual ~buffer_memory_strategy() = default;

    virtual void copy_from(const device_handle& in_device, vk::Buffer in_buffer, vk::DeviceMemory in_device_memory,
                           const void* in_data, vk::DeviceSize in_size,
                           vk::CommandBuffer in_command_buffer) = 0;

    static std::unique_ptr<buffer_memory_strategy> create(vk::MemoryPropertyFlags in_memory_props);
};

class host_visible_strategy : public buffer_memory_strategy {
public:
	virtual void copy_from(const device_handle& in_device, vk::Buffer in_buffer, vk::DeviceMemory in_device_memory, const void* in_data, vk::DeviceSize in_size, vk::CommandBuffer in_command_buffer) override;
};

class device_local_strategy : public buffer_memory_strategy {
public:
	virtual void copy_from(const device_handle& in_device, vk::Buffer in_buffer, vk::DeviceMemory in_device_memory, const void* in_data, vk::DeviceSize in_size, vk::CommandBuffer in_command_buffer) override;
};
