﻿#include "renderer_shader.h"

#include <fstream>
#include <spdlog/spdlog.h>

#include "renderer.h"

renderer_shader::~renderer_shader() {
	destroy();
}

bool renderer_shader::load(const std::filesystem::path& in_file_path, vk::Device in_device) {
	if (!in_device)
		return false;
	destroy();
	device = in_device;
	file_path = in_file_path;

	// 读取着色器文件

	std::ifstream file(in_file_path.c_str(), std::ios::binary | std::ios::ate);
	if (!file.is_open()) {
		spdlog::error("无法打开着色器文件: {}", in_file_path.generic_string());
		return false;
	}
	const auto file_size = file.tellg();
	std::vector<uint8_t> buffer(file_size);
	file.seekg(0);
	file.read(reinterpret_cast<char*>(buffer.data()), file_size);
	file.close();

	auto result = create_shader_module(buffer);
	if (!result) {
		spdlog::error("创建着色器模块失败");
		return false;
	}
	spdlog::info("加载着色器: {}", in_file_path.generic_string());
	return true;
}

void renderer_shader::destroy() {
	if (shader_module) {
		device.destroyShaderModule(shader_module);
		shader_module = nullptr;
		spdlog::info("销毁着色器: {}", file_path.generic_string());
	}
}

bool renderer_shader::create_shader_module(const std::vector<uint8_t>& in_code) {
	vk::ShaderModuleCreateInfo create_info{};
	create_info.setCodeSize(in_code.size());
	create_info.setPCode(reinterpret_cast<const uint32_t*>(in_code.data()));

	shader_module = device.createShaderModule(create_info);
	return shader_module;
}
