#pragma once
#include <spdlog/spdlog.h>
#include <vulkan/vulkan.hpp>

#include "renderer_types.h"

class renderer_swapchain {
public:
    struct create_info {
        vk::PresentModeKHR present_mode = vk::PresentModeKHR::eMailbox;             // 呈现模式
        vk::Format format               = vk::Format::eB8G8R8A8Srgb;                // 请求的图像格式
        vk::ColorSpaceKHR color_space   = vk::ColorSpaceKHR::eSrgbNonlinear;        // 色彩空间
        vk::ImageUsageFlags image_usage = vk::ImageUsageFlagBits::eColorAttachment; // 图像用途
        vk::Extent2D extent             = vk::Extent2D(800, 600);      // 交换链图像大小
        uint32_t min_image_count        = 2;                                        // 最小缓冲区数量
    };

    renderer_swapchain() {
    }

    ~renderer_swapchain() {
        cleanup();
    }

    bool resize(const vk::Extent2D& in_size) {
        cleanup();
        info.extent = in_size;
        return init_swapchain(device, surface, info);
    }

    [[nodiscard]] const auto& get_swapchain() const { return swapchain; }
    [[nodiscard]] const auto& get_images() const { return images; }
    [[nodiscard]] const auto& get_image_views() const { return image_views; }
    [[nodiscard]] const auto& get_create_info() const { return info; }
    [[nodiscard]] const auto& get_surface() const { return surface; }
    [[nodiscard]] const auto& get_device() const { return device; }

    bool init_swapchain(const device_handle& in_device, const vk::SurfaceKHR& in_surface, const create_info& in_info);

private:
    bool create_image_views(const vk::Device& device, vk::Format format);

    void cleanup();

private:
    device_handle device;
    create_info info;
    vk::SurfaceKHR surface;
    vk::SwapchainKHR swapchain;
    std::vector<vk::Image> images;
    std::vector<vk::ImageView> image_views;
};
