#include "window_manager.h"

#include <GLFW/glfw3.h>
#include <spdlog/spdlog.h>

#include "renderer_window.h"
#include "renderer/renderer.h"

window_manager* s_window_manager;

void glfw_error_callback(int error, const char* description) {
    spdlog::error("GLFW错误: {}, {}", error, description);
}

window_manager::~window_manager() {
    windows.clear();
}

bool window_manager::init() {
    glfwSetErrorCallback(glfw_error_callback);
    auto glfw_version = glfwGetVersionString();
    spdlog::info("GLFW 版本: {}", glfw_version);
    if (glfwInit() != GLFW_TRUE) {
        spdlog::error("初始化 GLFW 失败");
        return false;
    }
    glfwWindowHint(GLFW_CLIENT_API, GLFW_NO_API);
    spdlog::info("初始化 GLFW 成功");
    return true;
}

void window_manager::destroy() {
    windows.clear();
    glfwTerminate();
    spdlog::info("销毁 GLFW");
}

std::weak_ptr<renderer_window> window_manager::create_window(const window_desc& in_desc) {
    auto window_ptr = std::make_shared<renderer_window>(in_desc);
    windows.push_back(window_ptr);
    return window_ptr;
}

void window_manager::destroy_window(renderer_window* in_window) {
    const auto it = std::ranges::find_if(windows, [in_window](const auto& window) {
        return window.get() == in_window;
    });
    if (it != windows.end()) {
        windows.erase(it);
    }
}

void window_manager::update() {
    glfwPollEvents();
}

void window_manager::on_window_close() {
}

window_manager* aorii::get_window_manager() {
    return s_window_manager;
}

bool aorii::create_window_manager() {
    s_window_manager = new window_manager();
    if (!s_window_manager->init()) {
        delete s_window_manager;
        s_window_manager = nullptr;
        spdlog::error("创建窗口管理器失败");
        return false;
    }
    return true;
}

void aorii::destroy_window_manager() {
    s_window_manager->destroy();
    delete s_window_manager;
}

bool aorii::should_exit() {
    return s_window_manager->should_exit();
}
