#pragma once
#include "renderer_window.h"

class renderer_window;

class window_manager {
    friend class manager_window_event_listener;
public:
    ~window_manager();

    bool init();
    void destroy();

    std::weak_ptr<renderer_window> create_window(const window_desc& in_desc);
    void destroy_window(renderer_window* in_window);

    std::weak_ptr<renderer_window> get_main_window() {
        return windows.empty() ? std::weak_ptr<renderer_window>() : windows.front();
    }
    [[nodiscard]] const auto& get_windows() const { return windows; }
    void update();

    [[nodiscard]] bool should_exit() const {
        return windows.empty() || windows[0]->should_close();
    }
private:
    void on_window_close();
    std::vector<std::shared_ptr<renderer_window>> windows;
};

namespace aorii {
    window_manager* get_window_manager();
    bool create_window_manager();
    void destroy_window_manager();

    bool should_exit();
}
